package net.greenfieldtech.cloudonixsdk.cloudonixdialer.activities;

import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.Intent;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.media.AudioManager;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.Nullable;
import android.support.design.widget.Snackbar;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import butterknife.BindView;
import butterknife.OnClick;
import net.greenfieldtech.cloudonixsdk.api.interfaces.IVoIPObserver;
import net.greenfieldtech.cloudonixsdk.api.models.SDKConstants;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.R;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.dialogs.AddCallDialogListener;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.dialogs.CallInCallDialog;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.dialogs.SwitchDialogInterface;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.NetManager;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.SdkFacade;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.Utils;
import net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils.VoIPClient;

public class CallActivity extends BaseActivity implements
    AddCallDialogListener, SensorEventListener, SwitchDialogInterface {

  protected static final String TAG = "CallActivity";
  private static final int NUMBER_LENGTH_LIMIT = 32;

  private static int MEDIA_TYPE_EARPIECE = 1;
  private static int MEDIA_TYPE_SPEAKER = 2;
  private static int MEDIA_TYPE_BT = 3;

  private final static float PROXIMITY_DISTANCE_FAR_IN_CM = (float) 5;
  public static final int ON_CALL = 1;
  public static final int CALL_HANGUP = 1;
  public static final int CALL_PROXIMITY_OFF = 2;

  @BindView(R.id.root)
  LinearLayout root;
  @BindView(R.id.tv_current_call_number)
  TextView tvCurrentCallNumber;
  @BindView(R.id.tv_second_call_number)
  TextView tvSecondCallNumber;
  @BindView(R.id.tv_call_duration)
  TextView tvCallDuration;
  @BindView(R.id.tv_mute)
  TextView tvMute;
  @BindView(R.id.tv_speaker)
  TextView tvSpeaker;
  @BindView(R.id.btn_hold_unhold)
  ImageView btnHoldUnhold;
  @BindView(R.id.tv_hold_unhold)
  TextView tvHoldUnhold;
  @BindView(R.id.iv_second_contact)
  ImageView ivSecondContact;
  @BindView(R.id.btn_speaker)
  ImageView btnSpeaker;
  @BindView(R.id.btn_mute)
  ImageView btnMute;
  @Nullable
  @BindView(R.id.bluetooth)
  Button mBluetoothButton;

  private CallInCallDialog callInCallDialog;
  private VoIPClient voIPClient;
  private BluetoothAdapter mBluetoothAdapter;
  private SensorManager mSensorManager;
  private AudioManager mAudioManager;
  private Sensor mProximity;
  private Handler durationHandler;
  private Runnable durationRunnable;

  private boolean isMute;
  private boolean isHold;
  private boolean isSpeaker;
  private boolean isProximityOn;
  private boolean isPaused;

  private String currentCallId;
  private String secondCallId;
  private String currentCallNumber;
  private String secondCallNumber;
  private String keypadNumber = "";

  @Override
  protected int getLayoutResource() {
    return R.layout.activity_call;
  }

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    voIPClient = VoIPClient.getInstance();

    currentCallId = getIntent().getStringExtra("call_id");
    currentCallNumber = getIntent().getStringExtra("call_number");

    Log.d("TAG", "onCreate CallId: " + currentCallId);

    if (currentCallId == null
        && voIPClient.getCalls() != null
        && voIPClient.getCalls().length != 0) {
      currentCallId = voIPClient.getCalls()[0].key;
      currentCallNumber = voIPClient.getCalls()[0].url;
    }

    if (currentCallId == null) {
      if (isTaskRoot()) {
        startActivity(new Intent(this, MainActivity.class));
      }
      finish();
    }

    tvCurrentCallNumber.setText(currentCallNumber);

    mSensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
    mProximity = mSensorManager.getDefaultSensor(Sensor.TYPE_PROXIMITY);

//        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
    mAudioManager = (AudioManager) getSystemService(AUDIO_SERVICE);

    fixCurrentState(); // Call it only when creating activity!!

    durationHandler = new Handler();
    durationRunnable = new Runnable() {
      @Override
      public void run() {
        int duration = Utils.getDuration(voIPClient.getStatistics(currentCallId));
        tvCallDuration.setText(Utils.convertTime(duration));
        durationHandler.postDelayed(this, 1000);
      }
    };
    durationHandler.post(durationRunnable);

//        if (enableBluetooh()) {
//            Button bluetoothButton = (Button)findViewById(R.id.buttonBluetooth);
//            bluetoothButton.setText("Disable Bluetooth");
//            mBluetooth = true;
  }

//        mAddCall = (Button) findViewById(R.id.buttonAddCall);
//        mAddCall.setOnClickListener(new View.OnClickListener() {
//            @Override
//            public void onClick(View v) {
//                String currentCallNumber = ((EditText) findViewById(R.id.editTextAddCall)).getText().toString();
//
//                voIPClient.localHold(SdkFacade.findActiveCall(), true);
//
//                if (currentCallNumber.length() > 0) {
//                    voIPClient.dial(currentCallNumber);
//                }
//
//            }
//
//        });

//    @OnClick(R.id.btn_delete)
//    void delete() {
//        if (keypadNumber.length() > 0) {
//            keypadNumber = keypadNumber.substring(0, keypadNumber.length() - 1);
//            tvKeypadNumber.setText(keypadNumber);
//        }
//    }

//    @OnLongClick(R.id.btn_delete)
//    boolean clear() {
//        keypadNumber = "";
//        tvKeypadNumber.setText(keypadNumber);
//        return false;
//    }

  private void fixCurrentState() {
    // Fix 'hold / unhold' button state
    if (voIPClient.getCallState(currentCallId) == IVoIPObserver.CallState.CALL_STATE_LOCALHOLD) {
      onHoldUnhold();
      callback.onCallState(currentCallId, IVoIPObserver.CallState.CALL_STATE_LOCALHOLD, "");
    }

    // Fix speaker state
    if (voIPClient.isSpeakerOn()) {
      onSpeaker();
    }
  }

  @Override
  protected void onResume() {
    VoIPClient.getInstance().addCallbacksListener(callback);
    isPaused = false;
    super.onResume();
    mSensorManager.registerListener(this, mProximity, SensorManager.SENSOR_DELAY_NORMAL);
    durationHandler.post(durationRunnable);
    if (VoIPClient.getInstance().getCallState(currentCallId) == null) {
      VoIPClient.getInstance().hangup(currentCallId);
      doOnHangup();
    }

    currentCallId = SdkFacade.findActiveCall(currentCallId);

    Log.i(TAG, "+++Found callId:" + currentCallId);

    String callUrl = "";

    if (currentCallId != null) {
      callUrl = VoIPClient.getInstance().getCallUrl(currentCallId);
    }

    if (mBluetoothButton != null) {
      mBluetoothButton.setVisibility(voIPClient.isBluetoothOn() ? View.VISIBLE : View.GONE);
    }

    if (!TextUtils.isEmpty(callUrl)) {
      tvCurrentCallNumber.setText(callUrl);
    }
  }

  protected void onPause() {
    durationHandler.removeCallbacks(durationRunnable);
    voIPClient.removeCallbacksListener(callback);
    isPaused = true;
    super.onPause();
    mSensorManager.unregisterListener(this);
  }

  @Override
  public void onBackPressed() {
//        if (keypadContainer.getVisibility() == View.VISIBLE) {
//            hideKeypad();
//        } else {
    doOnHangup();
    super.onBackPressed();
//        }
  }

  @OnClick(R.id.iv_second_contact)
  void switchToSecondCall() {
    callInCallDialog = CallInCallDialog.newInstance("Name - " + secondCallNumber, secondCallNumber);
    callInCallDialog.show(getSupportFragmentManager(), "CallInCallDialog");
  }

  @OnClick(R.id.btn_hangup)
  void onHangup() {
    Log.i(TAG, "onHangup pressed. CallID: " + currentCallId);
    doOnHangup();
  }

  private void doOnHangup() {
    voIPClient.hangup(currentCallId);
    voIPClient.removeForegroundService();
    if (isTaskRoot()) {
      startActivity(new Intent(this, MainActivity.class));
    }
    finish();
  }

  @OnClick(R.id.btn_mute)
  void mute() {
    voIPClient.setMute(!isMute);
//        voIPClient.setMuteByKey(currentCallId, !isMute);
    isMute = !isMute;
    tvMute.setText(isMute ? "UnMute" : "Mute");
    btnMute.setSelected(isMute);
  }

  @OnClick(R.id.btn_hold_unhold)
  void onHoldUnhold() {
    if (NetManager.isConnected(this)) {
      if (!isHold) {
        Log.i(TAG, "onHold pressed");
        voIPClient.localHold(currentCallId, !isHold);
        tvHoldUnhold.setText("Unhold");
      } else {
        Log.i(TAG, "onUnhold pressed");
        voIPClient.localHold(currentCallId, !isHold);
        tvHoldUnhold.setText("Hold");
      }
      btnHoldUnhold.setEnabled(false);
    } else {
      Snackbar.make(root, "No internet connection", Snackbar.LENGTH_SHORT).show();
    }
  }

  @OnClick(R.id.btn_speaker)
  void onSpeaker() {
    Log.i(TAG, "onSpeaker pressed");

    isBluetoothEnable = false;

    if (!isSpeaker) {
      voIPClient.setSpeaker(true);
      tvSpeaker.setText("Ear");
    } else {
      voIPClient.setSpeaker(false);
      tvSpeaker.setText("Speaker");
    }
    isSpeaker = !isSpeaker;
  }

  private boolean isBluetoothEnable = false;

  @Nullable
  @OnClick(R.id.bluetooth)
  void onBluetooth(View v) {
    Log.i(TAG, "onBluetooth pressed");
    if (isBluetoothEnable) {
      voIPClient.setBluetooth(false);
    } else {
      voIPClient.setBluetooth(true);
    }
    isBluetoothEnable = !isBluetoothEnable;
  }

//    private boolean enableBluetooh() {
//        if ((mBluetoothAdapter != null) && (mBluetoothAdapter.getState() == BluetoothAdapter.STATE_OFF
//                || mBluetoothAdapter.getState() == BluetoothAdapter.STATE_TURNING_OFF)) {
//            Intent intentBluetooth = new Intent();
//            intentBluetooth.setAction(android.provider.Settings.ACTION_BLUETOOTH_SETTINGS);
//            startActivity(intentBluetooth);
//        } else {
//            if (!mAudioManager.isBluetoothScoOn()) {
//                boolean shouldConnect = false;
//                if (mBluetoothAdapter != null) {
//                    for (BluetoothDevice device : mBluetoothAdapter.getBondedDevices()) {
//                        if (device.getBondState() != BluetoothDevice.BOND_NONE) {
//                            // one device is available for connection
//                            shouldConnect = true;
//                            break;
//                        }
//                    }
//                }
//                if (shouldConnect) {
//                    voIPClient.setBluetooth(true);
//                    return true;
//                } else {
//                    Intent intentBluetooth = new Intent();
//                    intentBluetooth.setAction(android.provider.Settings.ACTION_BLUETOOTH_SETTINGS);
//                    startActivity(intentBluetooth);
//                }
//            }
//        }
//        return false;
//    }

//    @OnClick(R.id.btn_keypad)
//    void showKeypad() {
//        if (callBtnContainer.getVisibility() == View.VISIBLE) {
//            callBtnContainer.setVisibility(View.GONE);
//            keypadContainer.setVisibility(View.VISIBLE);
//        }
//    }
//
//    @OnClick(R.id.btn_hide_keypad)
//    void hideKeypad() {
//        if (keypadContainer.getVisibility() == View.VISIBLE) {
//            keypadContainer.setVisibility(View.GONE);
//            callBtnContainer.setVisibility(View.VISIBLE);
//        }
//    }

//    private void sendDTMF(String key, char c) {
//        voIPClient.sendDTMF(key, c);
//    }

//    @OnClick({R.id.btn_1, R.id.btn_2, R.id.btn_3, R.id.btn_4, R.id.btn_5, R.id.btn_6,
//            R.id.btn_7, R.id.btn_8, R.id.btn_9, R.id.btn_0, R.id.btn_star, R.id.btn_pound})
//    void buttonClicked(View button) {
//        if (tvKeypadNumber.getText().length() < NUMBER_LENGTH_LIMIT) {
//            AudioManager am = (AudioManager) getSystemService(AUDIO_SERVICE);
//
//            try {
//                String viewTag = (String) button.getTag();
//                try {
//                    int d = Integer.parseInt(viewTag);
//                    keypadNumber = keypadNumber + d;
//                } catch (NumberFormatException e) {
//                    if (viewTag.equals("*")) {
//                        keypadNumber += "*";
//                    }
//                    if (viewTag.equals("#")) {
//                        keypadNumber += "#";
//                    }
//                }
//                tvKeypadNumber.setText(keypadNumber);
//            } catch (ClassCastException e) {
//
//            }
//        } else {
//            Snackbar.make(root, "Limit of number length", Snackbar.LENGTH_SHORT).show();
//        }

//        try {
//            String viewTag = (String) v.getTag();
//            try {
//                String normalized = Normalizer.normalize(viewTag, Normalizer.Form.NFD);
//                try {
//                    sendDTMF(findActiveCall(),(char) normalized.charAt(0));
//                } catch (IndexOutOfBoundsException ex) {
//
//                }
//
//            } catch (NumberFormatException e) {
//                if (viewTag.equals("*")) {
//                    sendDTMF(findActiveCall(),'*');
//                }
//
//                if (viewTag.equals("#")) {
//                    sendDTMF(findActiveCall(),'#');
//                }
//            }
//
//        } catch (ClassCastException e) {
//
//        }
//    }

  public void proximityOn() {
    isProximityOn = true;
    if (!isPaused) {
      startActivityForResult(new Intent(this, LockScreenActivity.class), ON_CALL);
    }
  }

  public void proximityOff() {
    isProximityOn = false;
  }

  @Override
  public void onActivityResult(int requestCode, int resultCode, Intent data) {
    if (requestCode == ON_CALL) {
      if (resultCode == CALL_HANGUP) {
        doOnHangup();
      }
    }
    super.onActivityResult(requestCode, resultCode, data);
  }

  @Override
  public void onSensorChanged(SensorEvent event) {
    if (voIPClient.getAudioRoute().equals(SDKConstants.AudioRoute.EARPIECE.toString())) {
      float distance = event.values[0];

      if (distance < PROXIMITY_DISTANCE_FAR_IN_CM) {
        proximityOn();
      } else {
        proximityOff();
      }
    }
  }

  @Override
  public void onAccuracyChanged(Sensor sensor, int accuracy) {

  }

  private IVoIPObserver callback = new IVoIPObserver() {

    @Override
    public void onSipStarted() {

    }

    @Override
    public void onSipStartFailed() {

    }

    @Override
    public void onRegisterState(RegisterState result, int expiry) {

    }

    @Override
    public void onCallState(String key, CallState callState, String contactUrl) {
      Log.i(TAG, "onCallState: Key: " + key + " State: " + callState);
      switch (callState) {
        case CALL_STATE_DISCONNECTED:
        case CALL_STATE_DISCONNECTEDDUETOBUSY:
        case CALL_STATE_DISCONNECTEDDUETOTIMEOUT:
        case CALL_STATE_DISCONNECTEDMEDIACHANGED:
        case CALL_STATE_DISCONNECTEDDUETONOMEDIA:
        case CALL_STATE_DISCONNECTEDDUETONETWORKCHANGE:
          Log.i(TAG, "onHangup received by call activity");

          int numberOfCalls = VoIPClient.getInstance().getNumberOfCalls();
          if (numberOfCalls == 0) {
            doOnHangup();
          } else {
            // TODO: check multiple calls
            currentCallId = voIPClient.getCalls()[0].key;
            currentCallNumber = voIPClient.getCalls()[0].url;
//                        voIPClient.localHold(currentCallId, false);
            Log.i(TAG, "===Found callId:" + currentCallId);
          }
          break;

        case CALL_STATE_LOCALHOLD:
          tvHoldUnhold.setText("Unhold");
          btnHoldUnhold.setEnabled(true);
          isHold = true;
          break;

        case CALL_STATE_MEDIAACTIVE:
          Log.d(TAG, "ohHold called");
          tvHoldUnhold.setText("Hold");
          btnHoldUnhold.setEnabled(true);
          if (key != currentCallId) {
            currentCallId = key;
            Log.i(TAG, "{{{{Found callId:" + currentCallId);
          }
          isHold = false;
          break;

        case CALL_STATE_INCOMING:
          secondCallNumber = contactUrl;
          secondCallId = key;
          break;

        default:
          if (currentCallId == null) {
            currentCallId = key;
            Log.i(TAG, "<<<<Found callId:" + currentCallId);
          }
          break;
      }

      fixSwitchingCallState();
    }

    @Override
    public void onLog(int level, String message) {

    }

    @Override
    public void onSipStopped() {
      voIPClient.removeForegroundService();
      if (isTaskRoot()) {
        startActivity(new Intent(CallActivity.this, MainActivity.class));
      }
      finish();
    }

    @Override
    public void onBluetoothState(BluetoothState bluetoothState) {

    }

    @Override
    public void onAudioRouteChange(AudioRoute route) {

    }

    @Override
    public void onBluetoothBtnState(BluetoothButtonsState state, String command, String eventName,
        Integer buttonID, Integer mode) {

    }

    @Override
    public void onNetworkLost() {

    }

    @Override
    public void onNetworkChanged() {

    }

    @Override
    public void onNetworkRegained() {

    }

    @Override
    public void onLicense(LicensingState state, String description) {

    }

    @Override
    public void onNATTypeDetected(NATType type) {

    }

  };

  private void fixSwitchingCallState() {
    checkCallsNumber();
    setCorrectNumbers();
  }

  private void setCorrectNumbers() {
    tvCurrentCallNumber.setText(currentCallNumber);
    tvSecondCallNumber.setText(secondCallNumber);
  }

  private void checkCallsNumber() {
    if (voIPClient.getNumberOfCalls() > 1) {
      showSecondCall(true);
    } else {
      showSecondCall(false);
      hideCallInCallDialog();
    }
  }

  private void showSecondCall(boolean isShow) {
    if (isShow && !TextUtils.isEmpty(secondCallNumber)) {
      ivSecondContact.setVisibility(View.VISIBLE);
      tvSecondCallNumber.setVisibility(View.VISIBLE);
    } else {
      ivSecondContact.setVisibility(View.GONE);
      tvSecondCallNumber.setVisibility(View.GONE);
    }
  }

  private void hideCallInCallDialog() {
    if (callInCallDialog != null) {
      callInCallDialog.dismiss();
    }
  }

  private void switchBetweenCalls() {
    swipeCalls();
    setCorrectNumbers();
  }

  private void swipeCalls() {
    String buff = currentCallId;
    currentCallId = secondCallId;
    secondCallId = buff;

    buff = currentCallNumber;
    currentCallNumber = secondCallNumber;
    secondCallNumber = buff;
  }

  String mAddCallNumber = "";

  @Override
  public void setNumber(String number) {

    if ((number == null) || (number.length() == 0)) {
      return;
    }
    mAddCallNumber = number;
    voIPClient.localHold(SdkFacade.findActiveCall(currentCallId), true);
    if (mAddCallNumber.length() > 0) {
      voIPClient.dial(mAddCallNumber);
    }
  }

  @Override
  public void hangupAndAnswerSecondCall() {
    voIPClient.hangup(currentCallId);
    voIPClient.answer(secondCallId);
    switchBetweenCalls();
  }

  @Override
  public void holdAndAnswerSecondCall() {
    voIPClient.localHold(currentCallId, true);
    // TODO Try to check if call was answered before
    voIPClient.answer(secondCallId);
    voIPClient.localHold(secondCallId, false);
    switchBetweenCalls();
  }

  @Override
  public void rejectSecondCall() {
    voIPClient.reject(secondCallId);
  }

}
